#include "MemoryListener.h"

#include <sstream>

#include <ArmarXCore/core/logging/Logging.h>
#include <RobotAPI/libraries/armem/core/ice_conversions.h>
#include <RobotAPI/libraries/armem/core/error.h>


namespace armarx::armem::client::util
{

    MemoryListener::MemoryListener()
    {
    }


    void
    MemoryListener::updated(const std::vector<data::MemoryID>& updatedSnapshotIDs) const
    {
        std::vector<MemoryID> bos;
        fromIce(updatedSnapshotIDs, bos);
        updated(bos);
    }


    void
    MemoryListener::updated(const std::vector<MemoryID>& updatedSnapshotIDs) const
    {
        std::stringstream error;

        for (const auto& [subscription, callbacks] : this->callbacks)
        {
            std::vector<MemoryID> matchingSnapshotIDs;

            for (const MemoryID& updatedSnapshotID : updatedSnapshotIDs)
            {
                try
                {
                    if (contains(subscription, updatedSnapshotID))
                    {
                        matchingSnapshotIDs.push_back(updatedSnapshotID);
                    }
                }
                catch (const armem::error::InvalidMemoryID& e)
                {
                    // Log to debug, but ignore otherwise
                    error << "Error when comparing subscribed ID " << subscription
                          << " with updated ID " << updatedSnapshotID << ":\n"
                          << e.what() << "\n\n";
                }
            }

            if (not matchingSnapshotIDs.empty())
            {
                for (auto& callback : callbacks)
                {
                    callback(subscription, matchingSnapshotIDs);
                }
            }
        }
        if (error.str().size() > 0)
        {
            ARMARX_VERBOSE << "The following issues were encountered during MemoryListener::" << __FUNCTION__ << "(): \n\n"
                           << error.str();
        }
    }


    void
    MemoryListener::subscribe(const MemoryID& id, Callback callback)
    {
        callbacks[id].push_back(callback);
    }


    void
    MemoryListener::subscribe(const MemoryID& subscriptionID, CallbackUpdatedOnly callback)
    {
        subscribe(subscriptionID, [callback](const MemoryID&, const std::vector<MemoryID>& updatedSnapshotIDs)
        {
            callback(updatedSnapshotIDs);
        });
    }


}
