#include "aron_conversions.h"

#include <RobotAPI/libraries/aron/common/aron_conversions.h>

#include <RobotAPI/libraries/ArmarXObjects/ObjectID.h>
#include <RobotAPI/libraries/ArmarXObjects/aron_conversions.h>

#include <RobotAPI/libraries/aron/common/aron_conversions/core.h>
#include <RobotAPI/libraries/aron/common/aron_conversions/stl.h>
#include <RobotAPI/libraries/aron/common/aron_conversions/armarx.h>


namespace armarx::armem::robot
{

    /* to be moved */
    void toAron(long& dto, const IceUtil::Time& time)
    {
        dto = time.toMicroSeconds();
    }


    /* Robot */

    void fromAron(const arondto::Robot& dto, Robot& bo)
    {
        // fromAron(dto.description, bo.description);
        fromAron(dto.state, bo.config);
    }

    void toAron(arondto::Robot& dto, const Robot& bo)
    {
        // toAron(dto.description, bo.description);
        toAron(dto.state, bo.config);
    }

    /* RobotDescription */

    void fromAron(const arondto::RobotDescription& dto, RobotDescription& bo)
    {
        aron::fromAron(dto.name, bo.name);
        fromAron(dto.xml, bo.xml);
    }

    void toAron(arondto::RobotDescription& dto, const RobotDescription& bo)
    {
        aron::toAron(dto.name, bo.name);
        toAron(dto.xml, bo.xml);
    }

    /* RobotState */

    void fromAron(const arondto::RobotState& dto, RobotState& bo)
    {
        bo.timestamp = dto.timestamp;
        bo.globalPose.matrix() = dto.globalPose;
        bo.jointMap            = dto.jointMap;
    }

    void toAron(arondto::RobotState& dto, const RobotState& bo)
    {
        dto.timestamp = bo.timestamp;
        dto.globalPose = bo.globalPose.matrix();
        dto.jointMap   = bo.jointMap;
    }

}  // namespace armarx::armem::robot


namespace armarx::armem
{

    void robot::fromAron(const arondto::ObjectClass& dto, RobotDescription& bo)
    {
        bo.name = aron::fromAron<armarx::ObjectID>(dto.id).str();
        fromAron(dto.articulatedSimoxXmlPath, bo.xml);
    }

    void robot::toAron(arondto::ObjectClass& dto, const RobotDescription& bo)
    {
        toAron(dto.id, ObjectID(bo.name));
        toAron(dto.articulatedSimoxXmlPath, bo.xml);
    }


    void robot::fromAron(const arondto::ObjectInstance& dto, RobotState& bo)
    {
        fromAron(dto.pose, bo);
    }

    void robot::toAron(arondto::ObjectInstance& dto, const RobotState& bo)
    {
        toAron(dto.pose, bo);
    }


    void robot::fromAron(const objpose::arondto::ObjectPose& dto, RobotState& bo)
    {
        bo.timestamp = dto.timestamp;
        bo.globalPose = dto.objectPoseGlobal;
        bo.jointMap = dto.objectJointValues;
    }

    void robot::toAron(objpose::arondto::ObjectPose& dto, const RobotState& bo)
    {
        dto.timestamp = bo.timestamp;
        dto.objectPoseGlobal = bo.globalPose.matrix();
        dto.objectJointValues = bo.jointMap;
    }

}  // namespace armarx::armem
