/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Long.h"

// ArmarX
#include "Int.h"

namespace armarx::aron::datanavigator
{
    /* constructors */
    LongNavigator::LongNavigator(const data::AronLongPtr& o, const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eLong, path),
        detail::NavigatorBase<data::AronLong, LongNavigator>(o)
    {
    }

    LongNavigator::LongNavigator(const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eLong, path)
    {
    }

    LongNavigator::LongNavigator(const long d, const Path& path) :
        LongNavigator(data::AronLongPtr(new data::AronLong(d)), path)
    {
    }

    /* operators */
    bool LongNavigator::operator==(const LongNavigator& other) const
    {
        const auto& otherAron = other.toAronLongPtr();
        if (this->aron->value != otherAron->value)
        {
            return false;
        }
        return true;
    }
    bool LongNavigator::operator==(const LongNavigatorPtr& other) const
    {
        if (!other)
        {
            return false;
        }
        return *this == *other;
    }


    /* static methods */
    LongNavigatorPtr LongNavigator::FromAronLongPtr(const data::AronLongPtr& aron)
    {
        return std::make_shared<LongNavigator>(aron);
    }

    data::AronLongPtr LongNavigator::ToAronLongPtr(const LongNavigatorPtr& navigator)
    {
        return navigator ? navigator->toAronLongPtr() : nullptr;
    }

    /* public member functions */
    data::AronLongPtr LongNavigator::toAronLongPtr() const
    {
        return aron;
    }


    /* virtual implementations */
    std::string LongNavigator::getName() const
    {
        return "data::AronLong";
    }

    bool LongNavigator::fullfillsType(const typenavigator::NavigatorPtr& type) const
    {
        const auto i = IntNavigator();
        return type->getDescriptor() == type::Descriptor::eLong || type->getDescriptor() == type::Descriptor::eTime || i.fullfillsType(type);
    }

    typenavigator::NavigatorPtr LongNavigator::recalculateType() const
    {
        return std::make_shared<typenavigator::LongNavigator>(getPath());
    }
}
