/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "NDArray.h"

namespace armarx::aron::typenavigator
{
    // constructors
    NDArrayNavigator::NDArrayNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eNDArray, path)
    {
    }

    NDArrayNavigator::NDArrayNavigator(const type::AronNDArrayPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eNDArray, path),
        detail::NavigatorBase<type::AronNDArray, NDArrayNavigator>(o)
    {
    }

    // operators
    bool NDArrayNavigator::operator==(const NDArrayNavigator& other) const
    {
        if (getMaybe() != other.getMaybe())
        {
            return false;
        }
        return getDimensions() == other.getDimensions() && getElementSize() == other.getElementSize() && getTypename() == other.getTypename();
    }

    type::AronNDArrayPtr NDArrayNavigator::toAronNDArrayPtr() const
    {
        return this->aron;
    }

    std::vector<int> NDArrayNavigator::getDimensions() const
    {
        return this->aron->dimensions;
    }

    std::string NDArrayNavigator::getTypename() const
    {
        return this->aron->typeName;
    }

    unsigned int NDArrayNavigator::getElementSize() const
    {
        return 0; //type->elementSize;
    }

    void NDArrayNavigator::setTypename(const std::string& s)
    {
        this->aron->typeName = s;
    }

    void NDArrayNavigator::setDimensions(const std::vector<int>& v)
    {
        this->aron->dimensions = v;
    }

    void NDArrayNavigator::addDimension(int i)
    {
        this->aron->dimensions.push_back(i);
    }

    void NDArrayNavigator::setElementSize(unsigned int i)
    {
        //type->elementSize = i;
    }

    // virtual implementations
    std::string NDArrayNavigator::getName() const
    {
        return "AronNDArrayType";
    }
}

